// server/index.js
const express = require('express');
const cors = require('cors');
const { PrismaClient } = require('@prisma/client');
const { hashPassword, verifyPassword } = require('./utils/authHelpers'); 
const jwt = require('jsonwebtoken');

// Uncomment if we decide to use the authenticateToken middleware
//const { authenticateToken } = require('./middleware/authenticateToken');

const app = express();
const prisma = new PrismaClient();

app.use(cors());
app.use(express.json());

// Use an environment variable for the JWT_SECRET
const JWT_SECRET = process.env.JWT_SECRET;

if (!JWT_SECRET) {
    console.error('JWT_SECRET is not set');
    process.exit(1);
}

// Utility function to create JWT token
const createToken = (userId, userType) => {
    return jwt.sign({ userId, userType }, JWT_SECRET, { expiresIn: '1h' });
};

// Registration endpoint for a student
app.post('/register/student', async (req, res) => {
    try {
        const { username, password } = req.body;
        const existingStudent = await prisma.student.findUnique({
            where: { username }
        });
        if (existingStudent) {
            return res.status(409).send('Username already exists');
        }
        const hashedPassword = await hashPassword(password);
        const student = await prisma.student.create({
            data: { username, password: hashedPassword }
        });
    
        //Token Generation
        const token = createToken(student.id, 'student');
        res.status(201).json({ token }); // Send the token in the response
        } catch (error) {
        res.status(500).send('Error registering student');
        }
    });
    
  // Registration endpoint for a teacher
    app.post('/register/teacher', async (req, res) => {
        try {
        const { username, password } = req.body;
        const existingTeacher = await prisma.teacher.findUnique({
            where: { username }
        });
        if (existingTeacher) {
            return res.status(409).send('Username already exists');
        }
        const hashedPassword = await hashPassword(password);
        const teacher = await prisma.teacher.create({
            data: { username, password: hashedPassword }
        });
    
        //Token Generation
        const token = createToken(teacher.id, 'teacher');
        res.status(201).json({ token }); // Send the token in the response
        } catch (error) {
        res.status(500).send('Error registering teacher');
        }
    });
    
  // Login endpoint for a student
    app.post('/login/student', async (req, res) => {
        try {
        const { username, password } = req.body;
        const student = await prisma.student.findUnique({
            where: { username }
        });
        if (!student || !(await verifyPassword(password, student.password))) {
            return res.status(401).send('Invalid credentials');
        }
        const token = createToken(student.id, 'student');
        res.json({ token }); // Send the token in the response
        } catch (error) {
        res.status(500).send('Error logging in');
        }
    });
    
  // Login endpoint for teachers
    app.post('/login/teacher', async (req, res) => {
        try {
        const { username, password } = req.body;
        const teacher = await prisma.teacher.findUnique({
            where: { username }
        });
        if (!teacher || !(await verifyPassword(password, teacher.password))) {
            return res.status(401).send('Invalid credentials');
        }
        const token = createToken(teacher.id, 'teacher');
        res.json({ token }); // Send the token in the response
        } catch (error) {
        res.status(500).send('Error logging in');
        }
    });
    
  // Start the server
    const PORT = process.env.PORT || 5000;
    app.listen(PORT, () => {
        console.log(`Server running on port ${PORT}`);
    });